/**
 * Specifies how multiple vector searches are combined.
 *
 * Uncommitted: This API is subject to change in the future.
 *
 * @category Full Text Search
 */
export declare enum VectorQueryCombination {
    /**
     * Indicates that multiple vector queries should be combined with logical AND.
     */
    AND = "and",
    /**
     * Indicates that multiple vector queries should be combined with logical OR.
     */
    OR = "or"
}
/**
 * Uncommitted: This API is subject to change in the future.
 *
 * @category Full Text Search
 */
export interface VectorSearchOptions {
    /**
     * Specifies how multiple vector queries should be combined.
     */
    vectorQueryCombination?: VectorQueryCombination;
}
/**
 * Represents a vector query.
 *
 * Uncommitted: This API is subject to change in the future.
 *
 * @category Full Text Search
 */
export declare class VectorQuery {
    private _fieldName;
    private _vector;
    private _numCandidates;
    private _boost;
    constructor(fieldName: string, vector: number[]);
    /**
     * @internal
     */
    toJSON(): any;
    /**
     * Adds boost option to vector query.
     *
     * Uncommitted: This API is subject to change in the future.
     *
     * @param boost A floating point value.
     */
    boost(boost: number): VectorQuery;
    /**
     * Adds numCandidates option to vector query. Value must be >= 1.
     *
     * Uncommitted: This API is subject to change in the future.
     *
     * @param numCandidates An integer value.
     */
    numCandidates(numCandidates: number): VectorQuery;
    /**
     * Creates a vector query.
     *
     * Uncommitted: This API is subject to change in the future.
     *
     * @param fieldName The name of the field in the JSON document that holds the vector.
     * @param vector List of floating point values that represent the vector.
     */
    static create(fieldName: string, vector: number[]): VectorQuery;
}
/**
 * Represents a vector search.
 *
 * Uncommitted: This API is subject to change in the future.
 *
 * @category Full Text Search
 */
export declare class VectorSearch {
    private _queries;
    private _options;
    constructor(queries: VectorQuery[], options?: VectorSearchOptions);
    /**
     * @internal
     */
    get queries(): VectorQuery[];
    /**
     * @internal
     */
    get options(): VectorSearchOptions | undefined;
    /**
     * Creates a vector search.
     *
     * Uncommitted: This API is subject to change in the future.
     *
     * @param query A vectory query that should be a part of the vector search.
     */
    static fromVectorQuery(query: VectorQuery): VectorSearch;
}
